/*
 * menus.c
 */

/*
 * yank  -  yet another NoteKeeper
 * Copyright (C) 1999 Michael Humann <m.hussmann@home.ins.de>
 *
 * This program is free software; you  can redistribute it and/or modify it
 * under the terms of  the GNU General  Public License as published  by the
 * Free Software Foundation;  either version 2 of  the License, or (at your
 * option) any later version.
 *
 * This program  is  distributed in the  hope  that it will  be useful, but
 * WITHOUT  ANY   WARRANTY;  without   even   the  implied    warranty   of
 * MERCHANTABILITY or    FITNESS FOR A PARTICULAR   PURPOSE.   See  the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write to the  Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "menus.h"

static GnomeUIInfo file_menu [] =
{
    GNOMEUIINFO_MENU_OPEN_ITEM(cb_file_load_dialog, NULL),    
    GNOMEUIINFO_MENU_PRINT_ITEM(cb_print_note, NULL),
    GNOMEUIINFO_MENU_SAVE_ITEM(cb_save_file, NULL),
    GNOMEUIINFO_MENU_SAVE_AS_ITEM(cb_file_save_dialog, NULL),
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_MENU_CLOSE_ITEM(cb_close, NULL),
    GNOMEUIINFO_MENU_EXIT_ITEM(cb_exit, NULL),
    GNOMEUIINFO_END
};

static GnomeUIInfo add_menu [] =
{
    {
        GNOME_APP_UI_ITEM, N_("_TextNote"),
        N_("Add a text note to the tree"),
        cb_new_text, NULL, NULL, 0, 0, 't', GDK_CONTROL_MASK
    },
    {
        GNOME_APP_UI_ITEM, N_("_CheckNote"),
        N_("Add a check note to the tree"),
        cb_new_check, NULL, NULL, 0, 0, 'c', GDK_CONTROL_MASK
    },
    {
        GNOME_APP_UI_ITEM, N_("T_odoNote"),
        N_("Add a todo note to the tree"),
        cb_new_todo, NULL, NULL, 0, 0, 'o', GDK_CONTROL_MASK
    },
    GNOMEUIINFO_END
};

static GnomeUIInfo sort_menu [] =
{
    {
        GNOME_APP_UI_ITEM, N_("list by date"),
        N_("Sort todo-list by date"),
        cb_sort_todo_date, NULL, NULL, 0, 0, (gchar)NULL, GDK_CONTROL_MASK
    },
    {
        GNOME_APP_UI_ITEM, N_("list by priority"),
        N_("Sort todo-list by priority"),
        cb_sort_todo_prio, NULL, NULL, 0, 0, (gchar)NULL, GDK_CONTROL_MASK
    },
    {
        GNOME_APP_UI_ITEM, N_("list by date& priority"),
        N_("Sort todo-list by date& priority"),
        cb_sort_todo_date_prio, NULL, NULL, 0, 0, (gchar)NULL, GDK_CONTROL_MASK
    },
    GNOMEUIINFO_END
};

static GnomeUIInfo modify_menu [] =
{
    {
        GNOME_APP_UI_ITEM, N_("into a TextNote"),
        N_("Convert current note to a textnote"), cb_modify_text, NULL,
        NULL, 0, 0, (gchar)NULL, GDK_CONTROL_MASK
    },
    {
        GNOME_APP_UI_ITEM, N_("into a CheckNote"),
        N_("Convert current note to a checknote"), cb_modify_check, NULL,
        NULL, 0, 0, (gchar)NULL, GDK_CONTROL_MASK
    },
    {
        GNOME_APP_UI_ITEM, N_("into a TodoNote"),
        N_("Convert current note to a todonote"), cb_modify_todo, NULL,
        NULL, 0, 0, (gchar)NULL, GDK_CONTROL_MASK
    },
    GNOMEUIINFO_END
};
static GnomeUIInfo delete_menu [] =
{
    {
        GNOME_APP_UI_ITEM, N_("note or subtree"),
        N_("Delete note(s) from the tree"), cb_open_delete_sel, NULL, NULL,
        0, 0, 'd', GDK_CONTROL_MASK
    },    
    {
        GNOME_APP_UI_ITEM, N_("finished items"),
        N_("Expire finished note(s) from the tree"),
        cb_delete_finished_notes, NULL, NULL, 0, 0, (gchar)NULL,
        GDK_CONTROL_MASK
    },
    {
        GNOME_APP_UI_ITEM, N_("all notes"),
        N_("Delete all note(s) from the tree"), cb_clear_tree, NULL, NULL,
        0, 0, (gchar)NULL, GDK_CONTROL_MASK
    },
    GNOMEUIINFO_END
};

static GnomeUIInfo edit_menu [] =
{
    GNOMEUIINFO_SUBTREE(N_("Sort"), sort_menu),
    GNOMEUIINFO_SUBTREE(N_("Transform"), modify_menu),
    GNOMEUIINFO_SUBTREE(N_("Delete"), delete_menu),
    GNOMEUIINFO_MENU_FIND_ITEM(cb_search_dialog, NULL),
    GNOMEUIINFO_MENU_PREFERENCES_ITEM(cb_pref_dialog, NULL),
    {
        GNOME_APP_UI_ITEM, N_("Save Preferences"),
        N_("Save preferences for yank"), cb_save_preferences, NULL, NULL,
        0, 0, (gchar)NULL, GDK_CONTROL_MASK
    },
    GNOMEUIINFO_END
};

static GnomeUIInfo help_menu [] =
{
/* FIXME */
/*   GNOMEUIINFO_HELP ("gnome-yank"), */
    GNOMEUIINFO_MENU_ABOUT_ITEM(cb_about, NULL),
    GNOMEUIINFO_END
};

static GnomeUIInfo menu [] =
{
    GNOMEUIINFO_MENU_FILE_TREE(file_menu),
    GNOMEUIINFO_MENU_EDIT_TREE(edit_menu),
    GNOMEUIINFO_SUBTREE(N_("A_dd"), add_menu),
    GNOMEUIINFO_MENU_HELP_TREE(help_menu),
    GNOMEUIINFO_END
};

static GnomeUIInfo toolbar [] =
{
    GNOMEUIINFO_ITEM_STOCK(N_("TextNote"), N_("Add a Note"), cb_new_text,
                           GNOME_STOCK_PIXMAP_ADD),
    GNOMEUIINFO_ITEM_STOCK(N_("CheckNote"), N_("Add a CheckNote"),
                           cb_new_check, GNOME_STOCK_PIXMAP_ADD),
    GNOMEUIINFO_ITEM_STOCK(N_("TodoNote"), N_("Add a TodoNote"), cb_new_todo,
                           GNOME_STOCK_PIXMAP_ADD),
    GNOMEUIINFO_ITEM_STOCK(N_("Delete"), N_("Delete a note or a subtree"),
                           cb_open_delete_sel, GNOME_STOCK_PIXMAP_REMOVE),
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_ITEM_STOCK(N_("Find"), N_("Find in notes"), cb_search_dialog,
                           GNOME_STOCK_PIXMAP_SEARCH),
    GNOMEUIINFO_ITEM_STOCK(N_("Print"), N_("Print the current note"),
                           cb_print_note, GNOME_STOCK_PIXMAP_PRINT),
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_ITEM_STOCK(N_("Quit"), N_("Quit yank"), cb_exit,
                           GNOME_STOCK_PIXMAP_QUIT),
    GNOMEUIINFO_END
};

/* ------------------------------------------------------ */
/*
 * create the menus
 */

void
yank_install_menus_and_toolbar(GtkWidget* app)
{
    gnome_app_create_toolbar_with_data(GNOME_APP(app), toolbar, app);
    gnome_app_create_menus_with_data(GNOME_APP(app), menu, app);
    gnome_app_install_menu_hints(GNOME_APP(app), menu);
}

/* ------------------------------------------------------ */
