/*
 * preferences.c
 */

/*
 * yank  -  yet another NoteKeeper
 * Copyright (C) 1999 Michael Humann <m.hussmann@home.ins.de>
 *
 * This program is free software; you  can redistribute it and/or modify it
 * under the terms of  the GNU General  Public License as published  by the
 * Free Software Foundation;  either version 2 of  the License, or (at your
 * option) any later version.
 *
 * This program  is  distributed in the  hope  that it will  be useful, but
 * WITHOUT  ANY   WARRANTY;  without   even   the  implied    warranty   of
 * MERCHANTABILITY or    FITNESS FOR A PARTICULAR   PURPOSE.   See  the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write to the  Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "preferences.h"

/*
 * defaults
 */

gchar default_yank_crash_file[] = ".yank.crash";
gchar default_yank_def_file[] = ".yank.default";
gchar default_print_command[] = "lpr -Plp";

static yank_preferences preferences;
static GnomePropertyBox *pref_dialog;
static GtkWidget        *b_auto_save_on_exit;
static GtkWidget        *b_auto_save_preferences;
static GtkObject        *s_compression;
static GtkWidget        *b_wordwrap;
static GtkWidget        *b_linewrap;
static GtkWidget        *e_yank_crash_file;
static GtkWidget        *e_yank_def_file;
static GtkWidget        *e_print_command;

extern GtkWidget        *text_entry;

/* ------------------------------------------------------ */
/*
 * select preferences for yank
 */

void
cb_pref_dialog(GtkWidget *w, gpointer p)
{
    GtkWidget *title;
    
    pref_dialog = GNOME_PROPERTY_BOX(gnome_property_box_new());
    gtk_signal_connect(GTK_OBJECT(pref_dialog), "apply",
                       GTK_SIGNAL_FUNC(cb_pref_apply), pref_dialog);
    gtk_signal_connect(GTK_OBJECT(pref_dialog), "destroy",
                       GTK_SIGNAL_FUNC(cb_pref_cancel), pref_dialog);
    gtk_signal_connect(GTK_OBJECT(pref_dialog), "delete_event",
                       GTK_SIGNAL_FUNC(gtk_false), NULL);

    /*
     * misc
     */
    
    title = gtk_label_new(_("Files"));
    gtk_notebook_append_page(GTK_NOTEBOOK((pref_dialog)->notebook),
                             pref_files_p(), title);

    title = gtk_label_new(_("Notes"));
    gtk_notebook_append_page(GTK_NOTEBOOK((pref_dialog)->notebook),
                             pref_notes_p(), title);

    gtk_signal_connect(GTK_OBJECT(b_auto_save_on_exit), "toggled",
                       GTK_SIGNAL_FUNC(pref_changed), pref_dialog);
    gtk_signal_connect(GTK_OBJECT(s_compression), "value_changed",
                       GTK_SIGNAL_FUNC(pref_changed), pref_dialog);
    gtk_signal_connect(GTK_OBJECT(e_yank_crash_file), "changed",
                       GTK_SIGNAL_FUNC(pref_changed), pref_dialog);
    gtk_signal_connect(GTK_OBJECT(e_yank_def_file), "changed",
                       GTK_SIGNAL_FUNC(pref_changed), pref_dialog);
    gtk_signal_connect(GTK_OBJECT(b_auto_save_preferences), "toggled",
                       GTK_SIGNAL_FUNC(pref_changed), pref_dialog);
    gtk_signal_connect(GTK_OBJECT(b_wordwrap), "toggled",
                       GTK_SIGNAL_FUNC(pref_changed), pref_dialog);
    gtk_signal_connect(GTK_OBJECT(b_linewrap), "toggled",
                       GTK_SIGNAL_FUNC(pref_changed), pref_dialog);
    gtk_signal_connect(GTK_OBJECT(e_print_command), "changed",
                       GTK_SIGNAL_FUNC(pref_changed), pref_dialog);
    gtk_widget_grab_focus(b_auto_save_on_exit);
    gtk_widget_show_all(GTK_WIDGET(pref_dialog));
}

/* ------------------------------------------------------ */
/*
 * apply preferences
 */

void
cb_pref_apply(GnomePropertyBox *pref, gint page, gpointer data)
{
    preferences.auto_save_on_exit = gtk_toggle_button_get_active(
        GTK_TOGGLE_BUTTON(b_auto_save_on_exit));
    preferences.save_compression = GTK_ADJUSTMENT(s_compression)->value;
    if (preferences.yank_crash_file != NULL)
    {
        g_free(preferences.yank_crash_file);
    }
    preferences.yank_crash_file = g_strdup(gtk_entry_get_text(
        GTK_ENTRY(e_yank_crash_file)));
    if (preferences.yank_def_file != NULL)
    {
        g_free(preferences.yank_def_file);
    }
    preferences.yank_def_file = g_strdup(gtk_entry_get_text(
        GTK_ENTRY(e_yank_def_file)));
    preferences.auto_save_preferences = gtk_toggle_button_get_active(
        GTK_TOGGLE_BUTTON(b_auto_save_preferences));
    preferences.wordwrap = gtk_toggle_button_get_active(
        GTK_TOGGLE_BUTTON(b_wordwrap));
    gtk_text_set_word_wrap(GTK_TEXT(text_entry), preferences.wordwrap);    
    preferences.linewrap = gtk_toggle_button_get_active(
        GTK_TOGGLE_BUTTON(b_linewrap));
    gtk_text_set_line_wrap(GTK_TEXT(text_entry), preferences.linewrap);
    if (preferences.print_command != NULL)
    {
        g_free(preferences.print_command);
    }
    preferences.print_command = g_strdup(gtk_entry_get_text(
        GTK_ENTRY(e_print_command)));
    
    if (preferences.auto_save_preferences == TRUE)
    {
        save_preferences(get_preferences());
    }
}

/* ------------------------------------------------------ */
/*
 * cancel preferences
 */

void
cb_pref_cancel(GtkWidget *w, gpointer p)
{
    gtk_widget_destroy(GTK_WIDGET(p));
    p = NULL;
}

/* ------------------------------------------------------ */
/*
 * mixed preferences tab
 */

GtkWidget *
pref_files_p(void)
{
    GtkWidget *vbox;
    GtkWidget *vbox1;
    GtkWidget *frame;
    GtkWidget *hbox1;
    GtkWidget *label;
    GtkWidget *s_comp_factor;
    GtkWidget *button;
    gchar     *fname;
    
    vbox = gtk_vbox_new(FALSE, 2);
    gtk_widget_show(vbox);

    /*
     * saving
     */
    
    frame = gtk_frame_new(_("Load & Save"));
    gtk_widget_show(frame);
    gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, TRUE, 2);
    
    vbox1 = gtk_vbox_new(FALSE, 2);
    gtk_widget_show(vbox1);
    gtk_container_add(GTK_CONTAINER(frame), vbox1);
    
    b_auto_save_on_exit = gtk_check_button_new_with_label(
        _("Autosave notes on exit"));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b_auto_save_on_exit),
                                 preferences.auto_save_on_exit);
    gtk_widget_show(b_auto_save_on_exit);
    gtk_box_pack_start(GTK_BOX(vbox1), b_auto_save_on_exit, FALSE, TRUE, 2);
    
    hbox1 = gtk_hbox_new(FALSE, 0);
    gtk_widget_show(hbox1);
    gtk_box_pack_start(GTK_BOX(vbox1), hbox1, FALSE, TRUE, 2);
    
    label = gtk_label_new(_("Compression for saving (0 = off) "));
    gtk_widget_show(label);
    gtk_box_pack_start(GTK_BOX(hbox1), label, FALSE, TRUE, 2);
    
    s_compression = gtk_adjustment_new(preferences.save_compression, 0, 10,
                                       1, 1, 1);
    s_comp_factor = gtk_hscale_new(GTK_ADJUSTMENT(s_compression));
    gtk_scale_set_digits(GTK_SCALE(s_comp_factor), 0);
    gtk_widget_show(s_comp_factor);
    gtk_box_pack_start(GTK_BOX(hbox1), s_comp_factor, TRUE, TRUE, 2);
    
    hbox1 = gtk_hbox_new(FALSE, 0);
    gtk_widget_show(hbox1);
    gtk_box_pack_start(GTK_BOX(vbox1), hbox1, FALSE, TRUE, 2);

    label = gtk_label_new(_("Filename for crashes (SIGTERM) "));
    gtk_widget_show(label);
    gtk_box_pack_start(GTK_BOX(hbox1), label, FALSE, TRUE, 2);
    
    e_yank_crash_file = gtk_entry_new();
    if (preferences.yank_crash_file == NULL)
    {
        fname = g_strdup_printf("%s/%s", getenv("HOME"),
                                default_yank_crash_file);
        gtk_entry_set_text(GTK_ENTRY(e_yank_crash_file), fname);
        g_free(fname);
    }
    else
    {
        gtk_entry_set_text(GTK_ENTRY(e_yank_crash_file),
                           preferences.yank_crash_file);
    }
    gtk_object_set_user_data(GTK_OBJECT(e_yank_crash_file),
                             default_yank_crash_file);
    gtk_widget_show(e_yank_crash_file);
    gtk_box_pack_start(GTK_BOX(hbox1), e_yank_crash_file, TRUE, TRUE, 2);

    button = gtk_button_new_with_label(_("Browse"));
    gtk_widget_show(button);
    gtk_box_pack_start(GTK_BOX(hbox1), button, FALSE, TRUE, 2);
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
                       GTK_SIGNAL_FUNC(select_fname), e_yank_crash_file);
    
    button = gtk_button_new_with_label(_("Default"));
    gtk_widget_show(button);
    gtk_box_pack_start(GTK_BOX(hbox1), button, FALSE, TRUE, 2);
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
                       GTK_SIGNAL_FUNC(cb_yank_def_file), e_yank_crash_file);
    
    hbox1 = gtk_hbox_new(FALSE, 0);
    gtk_widget_show(hbox1);
    gtk_box_pack_start(GTK_BOX(vbox1), hbox1, FALSE, TRUE, 2);

    label = gtk_label_new(_("Default filename "));
    gtk_widget_show(label);
    gtk_box_pack_start(GTK_BOX(hbox1), label, FALSE, TRUE, 2);
    
    e_yank_def_file = gtk_entry_new();
    if (preferences.yank_def_file == NULL)
    {
        fname = g_strdup_printf("%s/%s", getenv("HOME"),
                                default_yank_def_file);
        gtk_entry_set_text(GTK_ENTRY(e_yank_def_file), fname);
        g_free(fname);
    }
    else
    {
        gtk_entry_set_text(GTK_ENTRY(e_yank_def_file),
                           preferences.yank_def_file);
    }
    gtk_object_set_user_data(GTK_OBJECT(e_yank_def_file),
                             default_yank_def_file);
    gtk_widget_show(e_yank_def_file);
    gtk_box_pack_start(GTK_BOX(hbox1), e_yank_def_file, TRUE, TRUE, 2);

    button = gtk_button_new_with_label(_("Browse"));
    gtk_widget_show(button);
    gtk_box_pack_start(GTK_BOX(hbox1), button, FALSE, TRUE, 2);
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
                       GTK_SIGNAL_FUNC(select_fname), e_yank_def_file);
    
    button = gtk_button_new_with_label(_("Default"));
    gtk_widget_show(button);
    gtk_box_pack_start(GTK_BOX(hbox1), button, FALSE, TRUE, 2);
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
                       GTK_SIGNAL_FUNC(cb_yank_def_file), e_yank_def_file);
    
    frame = gtk_frame_new(_("Preferences"));
    gtk_widget_show(frame);
    gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, TRUE, 2);
    
    vbox1 = gtk_vbox_new(FALSE, 2);
    gtk_widget_show(vbox1);
    gtk_container_add(GTK_CONTAINER(frame), vbox1);

    b_auto_save_preferences = gtk_check_button_new_with_label(
        _("Autosave preferences"));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b_auto_save_preferences),
                                 preferences.auto_save_preferences);
    gtk_widget_show(b_auto_save_preferences);
    gtk_box_pack_start(GTK_BOX(vbox1), b_auto_save_preferences,
                       FALSE, TRUE, 2);
    
    return (vbox);
}

/* ------------------------------------------------------ */
/*
 * return pointer to preferences
 */

inline
yank_preferences*
get_preferences(void)
{
    return (&preferences);
}

/* ------------------------------------------------------ */
/*
 * load yanks preferences
 */

void
load_preferences(void)
{
    gnome_config_push_prefix("/yank/Global/");
    
    preferences.auto_save_on_exit = gnome_config_get_int("auto save on exit");
    preferences.save_compression = gnome_config_get_int("save compression");
    preferences.yank_crash_file = g_strdup(gnome_config_get_string(
        "yank crash file"));
    if (preferences.yank_crash_file == NULL)
    {
        preferences.yank_crash_file = g_strdup_printf("%s/%s", getenv("HOME"),
                                                      default_yank_crash_file);
    }
    preferences.yank_def_file = g_strdup(gnome_config_get_string(
        "yank default file"));
    if (preferences.yank_def_file == NULL)
    {
        preferences.yank_def_file = g_strdup_printf("%s/%s", getenv("HOME"),
                                                      default_yank_def_file);
    }
    preferences.print_command = g_strdup(gnome_config_get_string(
        "print command"));
    if (preferences.print_command == NULL)
    {
        preferences.print_command = g_strdup(default_print_command);
    }
    preferences.auto_save_preferences = gnome_config_get_int
        ("auto save preferences");
    preferences.wordwrap = gnome_config_get_int("wordwrap");
    preferences.linewrap = gnome_config_get_int("linewrap");
    
    gnome_config_pop_prefix();
    gnome_config_sync();
}

/* ------------------------------------------------------ */
/*
 * save yanks preferences
 */

void
save_preferences(yank_preferences *prefs)
{
    gnome_config_push_prefix("/yank/Global/");
    
    gnome_config_set_int("auto save on exit", prefs->auto_save_on_exit);
    gnome_config_set_int("save compression", prefs->save_compression);
    gnome_config_set_string("yank crash file", prefs->yank_crash_file);
    gnome_config_set_string("yank default file", prefs->yank_def_file);
    gnome_config_set_int("auto save preferences",
                         prefs->auto_save_preferences);
    gnome_config_set_int("wordwrap", prefs->wordwrap);
    gnome_config_set_int("linewrap", prefs->linewrap);
    gnome_config_set_string("print command", prefs->print_command);
    
    gnome_config_pop_prefix();
    gnome_config_sync();
}

/* ------------------------------------------------------ */
/*
 * save from the menu
 */

void
cb_save_preferences(GtkWidget *w, gpointer p)
{
    save_preferences(get_preferences());
}

/* ------------------------------------------------------ */
/*
 * preferences have been changed 
 */

void
pref_changed(GtkWidget *w, GnomePropertyBox *prefs)
{
  gnome_property_box_changed(prefs);
}

/* ------------------------------------------------------ */

GtkWidget *
pref_notes_p(void)
{
    GtkWidget *vbox;
    GtkWidget *vbox1;
    GtkWidget *hbox1;
    GtkWidget *frame;
    GtkWidget *label;
    GtkWidget *button;

    vbox = gtk_vbox_new(FALSE, 2);
    gtk_widget_show(vbox);

    /*
     * editing
     */
    
    frame = gtk_frame_new(_("Editing"));
    gtk_widget_show(frame);
    gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, TRUE, 2);
    
    vbox1 = gtk_vbox_new(FALSE, 2);
    gtk_widget_show(vbox1);
    gtk_container_add(GTK_CONTAINER(frame), vbox1);
    
    b_wordwrap = gtk_check_button_new_with_label(
        _("Use wordwrap in note text"));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b_wordwrap),
                                 preferences.wordwrap);
    gtk_widget_show(b_wordwrap);
    gtk_box_pack_start(GTK_BOX(vbox1), b_wordwrap, FALSE, TRUE, 2);
    
    b_linewrap = gtk_check_button_new_with_label(
        _("Use linewrap in note text"));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b_linewrap),
                                 preferences.linewrap);
    gtk_widget_show(b_linewrap);
    gtk_box_pack_start(GTK_BOX(vbox1), b_linewrap, FALSE, TRUE, 2);
    
    frame = gtk_frame_new(_("Printing"));
    gtk_widget_show(frame);
    gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, TRUE, 2);
    
    vbox1 = gtk_vbox_new(FALSE, 2);
    gtk_widget_show(vbox1);
    gtk_container_add(GTK_CONTAINER(frame), vbox1);
    
    hbox1 = gtk_hbox_new(FALSE, 0);
    gtk_widget_show(hbox1);
    gtk_box_pack_start(GTK_BOX(vbox1), hbox1, FALSE, TRUE, 2);

    label = gtk_label_new(_("Print command "));
    gtk_widget_show(label);
    gtk_box_pack_start(GTK_BOX(hbox1), label, FALSE, TRUE, 2);
    
    e_print_command = gtk_entry_new();
    if (preferences.print_command == NULL)
    {
        gtk_entry_set_text(GTK_ENTRY(e_print_command), default_print_command);
    }
    else
    {
        gtk_entry_set_text(GTK_ENTRY(e_print_command),
                           preferences.print_command);
    }
    gtk_object_set_user_data(GTK_OBJECT(e_print_command),
                             default_print_command);
    gtk_widget_show(e_print_command);
    gtk_box_pack_start(GTK_BOX(hbox1), e_print_command, TRUE, TRUE, 2);

    button = gtk_button_new_with_label(_("Browse"));
    gtk_widget_show(button);
    gtk_box_pack_start(GTK_BOX(hbox1), button, FALSE, TRUE, 2);
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
                       GTK_SIGNAL_FUNC(select_fname), e_print_command);
    
    button = gtk_button_new_with_label(_("Default"));
    gtk_widget_show(button);
    gtk_box_pack_start(GTK_BOX(hbox1), button, FALSE, TRUE, 2);
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
                       GTK_SIGNAL_FUNC(cb_yank_def_cmd), e_print_command);
    
    return (vbox);
}

/* ------------------------------------------------------ */
/*
 * open fileselection
 */

void
select_fname(GtkWidget *w, gpointer p)
{
    GtkWidget *fs;
    gchar     *txt;
    
    txt = gtk_entry_get_text(GTK_ENTRY(p));
    fs = gtk_file_selection_new(_("Select filename"));
    if (txt != NULL)
    {
        gtk_file_selection_set_filename(GTK_FILE_SELECTION(fs), txt);
    }
    gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(fs));
    gtk_object_set_user_data(GTK_OBJECT(fs), p);
    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(fs)->ok_button),
                       "clicked", GTK_SIGNAL_FUNC(select_fname_ok), fs);
    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(fs)->cancel_button),
                       "clicked", GTK_SIGNAL_FUNC(select_fname_cancel), fs);
    gtk_widget_show(fs);
}

/* ------------------------------------------------------ */
/*
 * fileselection ok
 */

void
select_fname_ok(GtkWidget *w, gpointer p)
{
    GtkEntry *entry;
    
    entry = GTK_ENTRY(gtk_object_get_user_data(GTK_OBJECT(p)));
    gtk_entry_set_text(entry, gtk_file_selection_get_filename(
        GTK_FILE_SELECTION(p)));
    gtk_widget_destroy(GTK_WIDGET(p));
}

/* ------------------------------------------------------ */
/*
 * fileselection cancel
 */

void
select_fname_cancel(GtkWidget *w, gpointer p)
{
    gtk_widget_destroy(GTK_WIDGET(p));
}

/* ------------------------------------------------------ */
/*
 * set default file for crash, load
 */

void
cb_yank_def_file(GtkWidget *w, gpointer p)
{
    GtkEntry *entry;
    gchar    *fname1;
    gchar    *fname2;
    gchar    *def;
    
    entry = GTK_ENTRY(p);
    def = gtk_object_get_user_data(GTK_OBJECT(entry));
    fname1 = g_strdup_printf("%s/%s", getenv("HOME"), def);
    fname2 = gtk_entry_get_text(entry);
    if (strcmp(fname1, fname2))
    {
        gtk_entry_set_text(entry, fname1);
    }
    g_free(fname1);
}

/* ------------------------------------------------------ */

void
cb_yank_def_cmd(GtkWidget *w, gpointer p)
{
    GtkEntry *entry;
    gchar    *fname1;
    gchar    *fname2;
    
    entry = GTK_ENTRY(p);
    fname1 = gtk_object_get_user_data(GTK_OBJECT(entry));
    fname2 = gtk_entry_get_text(entry);
    if (strcmp(fname1, fname2))
    {
        gtk_entry_set_text(entry, fname1);
    }
}

/* ------------------------------------------------------ */
