/*
 * search.c
 */

/*
 * yank  -  yet another NoteKeeper
 * Copyright (C) 1999 Michael Humann <m.hussmann@home.ins.de>
 *
 * This program is free software; you  can redistribute it and/or modify it
 * under the terms of  the GNU General  Public License as published  by the
 * Free Software Foundation;  either version 2 of  the License, or (at your
 * option) any later version.
 *
 * This program  is  distributed in the  hope  that it will  be useful, but
 * WITHOUT  ANY   WARRANTY;  without   even   the  implied    warranty   of
 * MERCHANTABILITY or    FITNESS FOR A PARTICULAR   PURPOSE.   See  the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write to the  Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "search.h"

/* externals */
extern GtkWidget *note_tree;

/* globals */
static GtkWidget *r_list;

/* ------------------------------------------------------ */
/*
 * open the search dialog
 */

void
cb_search_dialog(GtkWidget *w, gpointer p)
{
    const gchar      *buttons[] =
    {
        GNOME_STOCK_BUTTON_OK,
        GNOME_STOCK_BUTTON_CLOSE,
        NULL
    };
    static GtkWidget *sdialog = NULL;
    gchar            *titles[] = { _("Results"), NULL };
    GtkWidget        *scrolledwindow;
    GtkWidget        *entry;
    GtkWidget        *app;
    GtkWidget        *frame;
    GtkWidget        *vbox;
    GtkWidget        *hbox;
    
    app = GTK_WIDGET(p);

    if (sdialog == NULL)
    {
        sdialog = gnome_dialog_newv(_("Find note(s)"), buttons);
        gnome_dialog_set_parent(GNOME_DIALOG(sdialog), GTK_WINDOW(app));
        gnome_dialog_set_default(GNOME_DIALOG(sdialog), 0);
        frame = gtk_frame_new(_("Find"));
        gtk_widget_show(frame);
        gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(sdialog)->vbox), frame, FALSE,
                           FALSE, 5);
        vbox = gtk_vbox_new(FALSE, 0);
        gtk_widget_show(vbox);
        gtk_container_add(GTK_CONTAINER(frame), vbox);
        hbox = gtk_hbox_new(FALSE, 0);
        gtk_widget_show(hbox);
        gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
        entry = gtk_entry_new();
        gtk_widget_show(entry);
        gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 5);
        scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
        gtk_container_set_border_width(GTK_CONTAINER(scrolledwindow), 5);
        gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
                                       GTK_POLICY_AUTOMATIC,
                                       GTK_POLICY_AUTOMATIC);
/*         gtk_widget_set_usize(scrolledwindow, 200, 100); */
        gtk_box_pack_start(GTK_BOX(vbox), scrolledwindow, FALSE, FALSE, 0);
        gtk_widget_show(scrolledwindow);
        r_list = gtk_clist_new_with_titles(1, titles);
        gtk_widget_show(r_list);
        gtk_clist_column_titles_passive(GTK_CLIST(r_list));
        gtk_signal_connect(GTK_OBJECT(r_list), "select_row", 
                           GTK_SIGNAL_FUNC(cb_search_row_selected), NULL); 
        gtk_container_add(GTK_CONTAINER(scrolledwindow), r_list);
        
        gnome_dialog_button_connect(GNOME_DIALOG(sdialog), 0,
                                    GTK_SIGNAL_FUNC(cb_search_ok), entry);
        gnome_dialog_button_connect(GNOME_DIALOG(sdialog), 1,
                                    GTK_SIGNAL_FUNC(cb_search_close), sdialog);
        gtk_signal_connect(GTK_OBJECT(sdialog), "destroy",
                           GTK_SIGNAL_FUNC(gtk_widget_destroyed), &sdialog);

        gnome_dialog_editable_enters(GNOME_DIALOG(sdialog),
                                     GTK_EDITABLE(entry));
        gtk_widget_grab_focus(entry);
        gtk_widget_show(sdialog);
    }
    else
    {
        g_assert(GTK_WIDGET_REALIZED(sdialog));
        gdk_window_show(sdialog->window);
        gdk_window_raise(sdialog->window);
    }
}

/* ------------------------------------------------------ */
/*
 * start searching
 */

void
cb_search_ok(GtkWidget *w, gpointer entry)
{
    static gchar *sstring = NULL;
    
    if (sstring)
    {
        g_free(sstring);
    }
    sstring = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
    gtk_clist_clear(GTK_CLIST(r_list));
    gtk_ctree_pre_recursive(GTK_CTREE(note_tree), NULL, (*search_tree_node),
                            sstring);
}

/* ------------------------------------------------------ */
/*
 * stop searching& kill the dialog
 */

void
cb_search_close(GtkWidget *w, gpointer p)
{
    gtk_widget_hide(GTK_WIDGET(p));
    gtk_widget_destroy(GTK_WIDGET(p));
}

/* ------------------------------------------------------ */
/*
 * search a single note
 */

void
search_tree_node(GtkCTree *tree, GtkCTreeNode *node, gpointer sstring)
{
    static gchar *txt[] = { NULL, NULL };
    note_data    *data;
    gint         row;
    
    data = gtk_ctree_node_get_row_data(tree, node);
    if (strstr(data->title, sstring) || strstr(data->text, sstring))
    {
        txt[0] = data->title;
        row = gtk_clist_append(GTK_CLIST(r_list), txt);
        gtk_clist_set_row_data(GTK_CLIST(r_list), row, data);
    }
}

/* ------------------------------------------------------ */
/*
 * an item in the list has been clicked
 */

void
cb_search_row_selected(GtkCList *list, gint row, gint col, GdkEvent *evt)
{
    note_data    *note;
    GtkCTreeNode *tnode;
    GtkCTreeNode *enode;
    
    if (evt)
    {
        clear_text_page();
        note = (note_data *)gtk_clist_get_row_data(GTK_CLIST(list), row);
        tnode = gtk_ctree_find_by_row_data(GTK_CTREE(note_tree), NULL, note);
        if (tnode != NULL)
        {
            enode = GTK_CTREE_ROW(tnode)->parent;
            while (enode)
            {
                gtk_ctree_expand(GTK_CTREE(note_tree), enode);
                enode = GTK_CTREE_ROW(enode)->parent;
            }
            gtk_ctree_select(GTK_CTREE(note_tree), tnode);
        }
    }
}

/* ------------------------------------------------------ */
